import sys
import csv

from Log import Logger
from PyQt5.QtWidgets import QApplication, QMainWindow, QAbstractItemView
from FileProxy import FileProxy
from Model import Model
from FileLoader import FileLoader
from PyQt5.QtCore import Qt, pyqtSignal, QModelIndex, pyqtSlot, QItemSelection, QItemSelectionModel
from PyQt5.QtWidgets import QFileDialog


from UI import Ui_MainWindow

class MyApp(QMainWindow, Ui_MainWindow):


    def __init__(self, parent=None):
        super(MyApp, self).__init__(parent)

        # Load UI widgets
        self.setupUi(self)

        self.log = Logger(self.logTextBox)
        #self.config = ConfigParser('default.ini')

        # Connect up the logger to the textbox
        self.clearLog.clicked.connect(self.logTextBox.clear)

        # Load the models and views
        model = Model(self)
        proxymodel = FileProxy(self)
        proxymodel.setSourceModel(model)

        # Pick the model
        self.mod = proxymodel
        self.inputTree.setModel(self.mod)
        
        # Needs to happen after the model is set
        self.selModel = self.inputTree.selectionModel()
        self.inputTree.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.inputTree.setSelectionMode(QAbstractItemView.MultiSelection)
                
        self.clearselection.clicked.connect(self.clearSelection)
        self.clearInput.clicked.connect(self.mod.clear)

        # Create a file loader that adds to the model
        fileLoader = FileLoader(self.mod, parent=self)
        self.inputTree.drop.connect(fileLoader.dropped)
        fileLoader.notifyProgress.connect(self.pbUpdate)

        # Hook up some buttons
        self.saveSummary.clicked.connect(self.saveStats)

        # Add a plot to the layout
        self.graphicsLayout.addModel(self.mod)
        self.graphicsLayout.addSelModel(self.selModel)

        self.graphicsLayout.updateView.connect(self.update)
        #self.graphicsLayout.setFixedHeight(2000)       

        # Plot button controls
        self.replot.clicked.connect(self.graphicsLayout.replot)
        self.savePlot.clicked.connect(self.graphicsLayout.exportView)

        self.progressBar.setProperty("value", 0)
        self.progressBar.setRange(0, 100)
        
        # Initialize the comboboxes
        self.combo1.insertItems(0, Model.drop_header_labels)
        self.combo1.setCurrentIndex(2)
        self.combo2.insertItems(0, Model.drop_header_labels)
        self.combo2.setCurrentIndex(3)
        self.combo3.insertItems(0, Model.drop_header_labels)
        self.combo3.setCurrentIndex(4)
        self.combo4.insertItems(0, Model.drop_header_labels)
        self.combo4.setCurrentIndex(5)

        self.combo1.currentIndexChanged.connect(self.mod.filterChanged)
        self.in1.textChanged.connect(self.mod.filterChanged)
        self.ex1.textChanged.connect(self.mod.filterChanged)

        self.combo2.currentIndexChanged.connect(self.mod.filterChanged)
        self.in2.textChanged.connect(self.mod.filterChanged)
        self.ex2.textChanged.connect(self.mod.filterChanged)

        self.combo3.currentIndexChanged.connect(self.mod.filterChanged)
        self.in3.textChanged.connect(self.mod.filterChanged)
        self.ex3.textChanged.connect(self.mod.filterChanged)

        self.combo4.currentIndexChanged.connect(self.mod.filterChanged)
        self.in4.textChanged.connect(self.mod.filterChanged)
        self.ex4.textChanged.connect(self.mod.filterChanged)

        
    def clearSelection(self):
        self.selModel.clearSelection()
              
    # Update the progressbar
    def pbUpdate(self, i):
        self.progressBar.setValue(i)

    def saveStats(self):
        (name, ext) = QFileDialog.getSaveFileName(
            None, 'Save Results File', None, "CSV files (*.csv);;All Files (*)")

        with open(name, 'w') as csvfile:

            spamwriter = csv.writer(
                csvfile, delimiter=',', quotechar='"', lineterminator='\n',
                quoting=csv.QUOTE_MINIMAL)

            spamwriter.writerow(self.mod.sourceModel().header_labels)

            for index in self.selModel.selectedRows():
                dataset = self.mod.data(index, Qt.UserRole)
                spamwriter.writerow(dataset.readDataAll(len(self.mod.sourceModel().header_labels)))
    
    def update(self):
        self.mod.layoutChanged.emit()


if __name__ == "__main__":
    app = QApplication(sys.argv)
    window = MyApp()

    window.show()
    sys.exit(app.exec_())
