import pyqtgraph as pg

from pyqtgraph import GraphicsWidget, LabelItem, ScatterPlotItem, Point, PlotDataItem, GraphicsWidgetAnchor
from pyqtgraph import ScatterPlotItem

from PyQt5 import QtCore, QtGui
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QBrush

import pyqtgraph.functions as fn

__all__ = ['LegendItem']

class LegendItem(GraphicsWidget, GraphicsWidgetAnchor):
    def __init__(self, size=None, offset=None):
        GraphicsWidget.__init__(self)
        GraphicsWidgetAnchor.__init__(self)
        self.setFlag(self.ItemIgnoresTransformations)
        self.layout = QtGui.QGraphicsGridLayout()
        self.setLayout(self.layout)
        self.items = []
        self.reset(size, offset)

    def setParentItem(self, p):
        ret = GraphicsWidget.setParentItem(self, p)
        if self.offset is not None:
            offset = Point(self.offset)
            anchorx = 1 if offset[0] <= 0 else 0
            anchory = 1 if offset[1] <= 0 else 0
            anchor = (anchorx, anchory)
            self.anchor(itemPos=anchor, parentPos=anchor, offset=offset)
        return ret
        
    def addItem(self, item, name):
        label = LabelItem(name, color='ffffff', justify='right')
        if isinstance(item, ItemSample):
            sample = item
        else:
            sample = ItemSample(item)        

        row = self.layout.rowCount()
        self.items.append((sample, label))
        self.layout.addItem(sample, row, 0)
        self.layout.addItem(label, row, 1)
        self.updateSize()
    

    def reset(self, size=None, offset=None):
        
        # Clear all items from the layout
        self.removeAllItem()

        self.items = []
        self.size = size
        self.offset = offset
        if size is not None:
            self.setGeometry(QtCore.QRectF(0, 0, self.size[0], self.size[1]))

        self.updateSize()

    def removeAllItem(self):
        for sample, label in self.items:
            self.layout.removeItem(sample)          # remove from layout
            sample.close()                          # remove from drawing
            self.layout.removeItem(label)
            label.close()

    def updateSize(self):
        if self.size is not None:
            return
            
        height = 0
        width = 0
        #print("-------")
        for sample, label in self.items:
            lw = label.width()
            lh = label.height()

            sw = sample.width()
            sh = sample.height()
            sw = 20
            sh = 20

            height += max(sh, lh) + 3
            width   = max(width, (sw + lw ))

        self.setGeometry(0, 0, width+25, height)
    
    def boundingRect(self):
        return QtCore.QRectF(0, 0, self.width(), self.height())
    
    def paint(self, p, *args):
        p.setPen(fn.mkPen(255,255,255,255))
        p.setBrush(fn.mkBrush(255,255,255,80))
        p.drawRect(self.boundingRect())

    def hoverEvent(self, ev):
        ev.acceptDrags(QtCore.Qt.LeftButton)
        
    def mouseDragEvent(self, ev):
        if ev.button() == QtCore.Qt.LeftButton:
            dpos = ev.pos() - ev.lastPos()
            self.autoAnchor(self.pos() + dpos)
        
class ItemSample(GraphicsWidget):
    ## Todo: make this more generic; let each item decide how it should be represented.
    def __init__(self, item,  *args, **kwargs):
        super(ItemSample, self).__init__(*args, **kwargs) 
        self.item = item
    
    def boundingRect(self):
        return QtCore.QRectF(0, 0, 20, 20)
        
    def paint(self, p, *args):
        opts = self.item.opts
        pen = opts['pen']
        color = pen.color()
        
        if 'brush' in opts:
            brush = opts['brush']
            if brush:
                color = brush.color()

        p.setBrush(QBrush(color))
        p.drawPolygon(QtGui.QPolygonF([QtCore.QPointF(4,4), QtCore.QPointF(16,4), QtCore.QPointF(16,16), QtCore.QPointF(4,16) ]    ))
        
        

        
        
        
        
